#!/usr/bin/python
from subprocess import Popen, PIPE
from optparse import OptionParser
import os,sys
import datetime,time
import logging
import commands

abs_path = os.path.abspath(sys.argv[0])
basename = os.path.basename(sys.argv[0])
dirname = abs_path.replace(basename, "")

usage="\n./%prog [options]"

parser = OptionParser(usage)
parser.add_option("-d", "--debug",   dest="debug", default=False, action='store_true',
                                    help="debug option to show detail commands to test")
parser.add_option("-i", "--install", dest="install", default=False, action='store_true',
                  help="install it in crontab as a current location")
parser.add_option("-l", "--list",    dest="showlist", default=False, action='store_true',
                  help="print list of monitoring items and path for history files")
(opts, args) = parser.parse_args()

loggingLevel=logging.INFO
if opts.debug:
    loggingLevel=logging.DEBUG
    
logging.basicConfig(format='%(levelname)s - %(funcName)s :%(message)s', level=loggingLevel) #filename="log.log"

_workdir= "/var/tmp"
_outdir = "/var/log/monitoring_system"
_sa_dir         = "/var/log/sa"
_sa_dir_archive = "%s/archive" % _sa_dir
_retention_days = 365
if opts.install:
    #0 */1 * * * /root/benchmark/system_monitoring/system_monitoring.py > /dev/null 2>&1
    #os.system("rm /var/log/monitoring_system/crontab_list.history /var/tmp/.crontab_list.prev")
    os.system("/usr/bin/crontab -l > /tmp/crontmp")
    doesExist=False
    for line in commands.getoutput("/bin/cat /tmp/crontmp").splitlines():
        if basename in line:
            doesExist=True

    if not doesExist:
        aCrontab="55 */1 * * * %s > /dev/null 2>&1" %(abs_path)
        os.system("/bin/cp /tmp/crontmp /tmp/crontmp.commit")
        os.system("/bin/echo '%s' >> /tmp/crontmp.commit" % aCrontab)
        os.system("/usr/bin/crontab /tmp/crontmp.commit >> /tmp/crontmp.commit.log 2>&1")
    #else: Could implement update crontab...
        
    sys.exit()  # no need to go through post process

    
if not os.path.exists(_outdir):
    os.system("/bin/mkdir %s" %_outdir)

if not os.path.exists(_sa_dir_archive):
    os.system("/bin/mkdir %s" %_sa_dir_archive)

def get_master_ini(section, para):
    return get_first_word("/usr/bp/bin/bputil -g %s %s ERROR /usr/bp/bpinit/master.ini" % (section, para))
        
# jobname will decide temp output file
# command will create temp file
def output_comparer(jobname, givenCommand, doCat=False):

    if doCat:
        filename=givenCommand
        if not os.path.exists(filename):
            return
        givenCommand="/bin/cat " + filename
        
    
    curr_outfile="%s/.%s.curr" %(_workdir,jobname)
    prev_outfile="%s/.%s.prev" %(_workdir,jobname)
    monitoring_outfile= "%s/%s.history" %(_outdir,jobname)
    if opts.showlist:
        print "%-62s  %s" %(givenCommand,monitoring_outfile)

    if not os.path.exists(prev_outfile):
        logging.debug("%s doesn't exist. Creating one." %prev_outfile)
        os.system("/bin/touch %s" % prev_outfile)

    ## Special treatment for history
    if jobname=="history":
        lines,finalLine=[],''
        curout=open(curr_outfile,'w')
        
        for line in get_printout("/bin/grep -v $^ /root/.bash_history"):
            if line[0:1] == "#":
                finalLine+=line+" "
            else:
                finalLine+=line
                lines.append(finalLine)
                finalLine=''

        for line in lines:
            words=line.split()
            if not len(words): continue
            theFirst=words[0]
            epoch,command=0,''
            if theFirst[0:1] == "#":
                epoch=int(theFirst[1:])
                command=line.split(' ',1)[1] # Do only one split as maximum
            else:
                command=line
                
            hepoch=time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(epoch))
            curout.write("%s   %s\n" %(hepoch,command))

        curout.close()
    else:
        theCommand=givenCommand + " > %s" %curr_outfile
        logging.debug(theCommand)
        os.system(theCommand)

    ## Make diff
    diffCommand="/usr/bin/diff %s %s" %(prev_outfile,curr_outfile)
    if "/usr/bp/bpinit/master.ini" in givenCommand:
        diffCommand="/usr/bin/diff %s %s | grep -Ev 'UpdateTime=|Status=Cleaned'" %(prev_outfile,curr_outfile)
        
    lines=commands.getoutput(diffCommand).splitlines()
    if len(lines):
        done_epoch=int(os.path.getmtime(curr_outfile)+0.5)
        done_hepoch = time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(done_epoch))
        
        theCommand="/bin/mv %s %s" %(curr_outfile,prev_outfile)
        logging.debug(theCommand)
        os.system(theCommand)
        for line in lines:
            if line[0:1] in (">","<"):
                if jobname=="history":
                    os.system("""echo \"%s\" >> %s""" % (line.replace("\"","\\\""),monitoring_outfile))
                else:
                    os.system("echo '%s  %s' >> %s" % (done_hepoch,line,monitoring_outfile))

    return

def list_files_comparer(listfiles):
    command="/bin/ls %s" % listfiles
    if "/bin/ls " in listfiles:
        command=listfiles
    for filename in commands.getoutput(command).splitlines():
        basefilename = os.path.basename(filename)
        output_comparer(basefilename,filename,True) # doCat=True
    
def main():

    list_files_comparer("/etc/sysconfig/network-scripts/ifcfg-*")
    list_files_comparer("/etc/sysconfig/network-scripts/route-*")
    list_files_comparer("/etc/cron.*/*")
    list_files_comparer("/bin/find /etc/ -maxdepth 1 -not -type d | /bin/grep cron")

    output_comparer("history",              "history")  
    output_comparer("crontab_l",            "/usr/bin/crontab -l")
    output_comparer("mount",                "/bin/mount")
    output_comparer("route",                "/sbin/route")

    output_comparer("platform_capabilities",     "/var/opt/unitrends/platform_capabilities",  True) # doCat=True
    output_comparer("master_ini",                "/usr/bp/bpinit/master.ini",                 True) # doCat=True
    output_comparer("etc_hosts",                 "/etc/hosts",                                True) # doCat=True
    output_comparer("pg_hba_conf",               "/usr/bp/data/pg_hba.conf",                  True) # doCat=True
    output_comparer("pg_service_conf",           "/usr/bp/data/pg_service.conf",              True) # doCat=True
    output_comparer("smb_conf",                  "/etc/samba/smb.conf",                       True) # doCat=True
    output_comparer("openvpn_inbound_at_target", "/etc/openvpn/inbound.conf",                 True) # doCat=True
    output_comparer("psql_history",              "/root/.psql_history",                       True) # doCat=True
    output_comparer("fstab",                     "/etc/fstab",                                True) # doCat=True
    output_comparer("persistent-net_rules",      "/etc/udev/rules.d/70-persistent-net.rules", True) # doCat=True
    target_name=get_master_ini("Replication","SyncTo")
    if target_name not in ["ERROR","vault"] and len(target_name):
        output_comparer("openvpn_%s_at_source" % target_name, "/etc/openvpn/%s.conf" % target_name, True) # doCat=True

    #######################
    #### sar metrics retention
    now = datetime.datetime.now()
    if opts.debug: # For debug, need to open 23 hours instead of 1 hour window
        if now.hour==1:  
            return
    else:
        if now.hour!=1:  
            return

    # /var/log/sa
    for filename in get_printout("/bin/ls %s/sa[0-9]*" % _sa_dir):
        mtime=datetime.datetime.fromtimestamp(os.path.getmtime(filename))
        if now.date()==mtime.date(): continue    #Do not proceed the file for today in progress
        
        dateString  = "%4d%02d%02d" %(mtime.year, mtime.month, mtime.day)
        gz_filename = "%s/sa.%s.gz" %(_sa_dir_archive,dateString)
        if os.path.exists(gz_filename): continue #Do not proceed if gzip file does exist 
        
        command="/bin/gzip -c %s > %s" %(filename,gz_filename)
        logging.debug("%s %s %s %s %s" %(mtime, command, now.date(), mtime.date(), now.date()==mtime.date()))
        if not opts.debug:
            os.system(command)

    # /var/log/sa/archive
    if int(get_first_word("/bin/ls %s/sa.*.gz 2>&1 | /bin/grep -v cannot | wc -l" % _sa_dir_archive)):
        earliest_date = (now-datetime.timedelta(days=_retention_days)).date()

        for filename in get_printout("/bin/ls %s/sa.*.gz" % _sa_dir_archive):
            dateString=filename.split(".")[1]
            filedate=datetime.date(int(dateString[0:4]),int(dateString[4:6]),int(dateString[6:8]))
            logging.debug("%s %s %s %s" %(filename,dateString,filedate,filedate>earliest_date))

            if filedate<earliest_date:
                command="/bin/rm %s" % filename
                if not opts.debug:
                    os.system(command)
            
    return

def get_printout(command, doSplit=True):

    if opts.debug: print "# " + command
    
    import commands
    if doSplit: # To make an array
        return commands.getoutput(command).splitlines()
    else :
        return commands.getoutput(command)
      
def get_nth_word(command, n):
   lines = get_printout(command)
   lineall = ""
   for line in lines:
       lineall += line

   if len(lineall)==0:
       return ""
   else:
       return lineall.split()[n]

def get_last_word(command):
   lines = get_printout(command)
   word =""
   for line in lines:
       words = line.split()
       word = words[len(words)-1]

   return word    

def get_first_word(command):
   return get_nth_word(command,0)

if __name__ == '__main__':

    main()

